// gcc -std=c99 -Wall -Wextra -pedantic -O3 cco.c -lm -lsndfile

#include <math.h>
#include <sndfile.h>

static const float h = 0.01;

static const float a[6][5] = 
  { {0, 0, 0, 0, 0}
  , {1./4, 0, 0, 0, 0}
  , {3./32, 9./32, 0, 0, 0}
  , {1932./2197, -7200./2197, 7296./2197, 0, 0}
  , {439./216, -8, 3680./513, -845./4104, 0}
  , {-8./27, 2, -3544./2565, 1859./4104, -11./40}
  };

static const float b[6] = {16./135, 0, 6656./12825, 28561./56430, -9./50, 2./55};

static float g;

static inline void f(float *out, const float v[4])
{
  float out0 = v[2];
  float out1 = v[3];
  float out2 = -expf(g * v[1] * v[1]) * v[0];
  float out3 = -expf(g * v[0] * v[0]) * v[1];
  out[0] = out0;
  out[1] = out1;
  out[2] = out2;
  out[3] = out3;
}

static const int s = 300;
static float samples[300 * 48000][2];

void rk4(float *v)
{
  float k[6][4];
  f(k[0], v);
  for (int p = 1; p < 6; ++p)
  {
    float u[4] = { 0, 0, 0, 0 };
    for (int q = 0; q < p; ++q)
      for (int j = 0; j < 4; ++j)
        u[j] += a[p][q] * k[q][j];
    for (int j = 0; j < 4; ++j)
      u[j] = v[j] + h * u[j];
    f(k[p], u);
  }
  float u[4] = { 0, 0, 0, 0 };
  for (int q = 0; q < 6; ++q)
    for (int j = 0; j < 4; ++j)
      u[j] += b[q] * k[q][j];
  for (int j = 0; j < 4; ++j)
    v[j] += h * u[j];
}

int main()
{
  float v[4] = { 0, 1, 1, 1 };
  for (int i = 0; i < s * 48000; ++i)
  {
    g = 2 * i / (float) (s * 48000);
    // rk4 integration
    rk4(v);
    // save oscs
    samples[i][0] = 0.25f * v[2];
    samples[i][1] = 0.25f * v[3];
  }
  SF_INFO info = { 0, 48000, 2, SF_FORMAT_WAV | SF_FORMAT_FLOAT, 0, 0 };
  SNDFILE *sf = sf_open("cco.wav", SFM_WRITE, &info);
  sf_writef_float(sf, &samples[0][0], s * 48000);
  sf_close(sf);
  return 0;
}
